@@ -1,10 +1,9 @@
-/* Copyright (c) 1996,1997, 1998, 1999, 2000, 2002 Thorsten Kukuk
+/* Copyright (c) 1996,1997, 1998, 1999, 2000, 2002, 2003, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
 
    The YP Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -386,6 +385,9 @@ create_file (char *fileName, char *dbmName, char *masterName,
 	{
 	  while (*cptr && *cptr != '\t')
 	    ++cptr;
+	  /* But a key should not end with a space.  */
+	  while (cptr[-1] == ' ')
+	    --cptr;
 	}
 
       *cptr++ = '\0';
@@ -481,7 +483,7 @@ dump_file (char *dbmName)
 	  perror (dbmName);
 	  exit (1);
 	}
-      printf ("%.*s %.*s\n",
+      printf ("%.*s\t%.*s\n",
 	      key.dsize, key.dptr,
 	      data.dsize, data.dptr);
       free (data.dptr);
@@ -497,7 +499,7 @@ dump_file (char *dbmName)
 	  perror (dbmName);
 	  exit (1);
 	}
-      printf ("%.*s %.*s\n",
+      printf ("%.*s\t%.*s\n",
 	      key.dsize, key.dptr,
 	      data.dsize, data.dptr);
       key = dbm_nextkey (dbm);
