@@ -1,10 +1,9 @@
-/* Copyright (c) 2000, 2001, 2002, 2003  Thorsten Kukuk
+/* Copyright (c) 2000, 2001, 2002, 2003, 2005  Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
 
    The YP Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,38 +22,40 @@
 #define _GNU_SOURCE
 
 #include <string.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif /* HAVE_ALLOCA_H */
 #include <unistd.h>
+#include <stdlib.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <time.h>
 #include "yp.h"
 #include "yp_db.h"
 #include "access.h"
 #include "ypserv_conf.h"
 #include "log_msg.h"
-
-extern volatile int children; /* ypserv.c  */
-extern int forked; /* ypserv.c  */
+#include "compat.h"
 
 bool_t
-ypproc_null_2_svc (void *argp __attribute__ ((unused)),
-		   void *result __attribute__ ((unused)),
+ypproc_null_2_svc (void *argp UNUSED, void *result UNUSED,
 		   struct svc_req *rqstp)
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_null() [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr),
 	       ntohs (rqhost->sin_port));
     }
 
-  if (!is_valid (rqstp, NULL, NULL))
+  if (is_valid (rqstp, NULL, NULL) < 1)
     return FALSE;
 
   return TRUE;
@@ -67,7 +68,7 @@ ypproc_domain_2_svc (domainname *argp, bool_t *result,
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_domain(\"%s\") [From: %s:%d]",
 	       *argp, inet_ntoa (rqhost->sin_addr),
 	       ntohs (rqhost->sin_port));
@@ -84,6 +85,11 @@ ypproc_domain_2_svc (domainname *argp, bool_t *result,
         log_msg ("\t-> Ignored (not a valid source host)");
       *result = FALSE;
       break;
+    case 0:
+      if (debug_flag)
+        log_msg ("\t-> Ignored (forbidden by securenets)");
+      *result = FALSE;
+      break;
     default:
       *result = TRUE;
     }
@@ -102,7 +108,7 @@ ypproc_domain_nonack_2_svc (domainname *argp, bool_t *result,
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_domain_nonack(\"%s\") [From: %s:%d]",
 	       *argp, inet_ntoa (rqhost->sin_addr),
 	       ntohs (rqhost->sin_port));
@@ -116,11 +122,16 @@ ypproc_domain_nonack_2_svc (domainname *argp, bool_t *result,
       /* Bail out and don't return any RPC value */
       return FALSE;
     case -2: /* Should not happen */
+      log_msg ("Map name not valid, this cannot happen???");
       return FALSE;
     case -1:
       if (debug_flag)
         log_msg ("\t-> Ignored (not a valid source host)");
       return FALSE;
+    case 0:
+      if (debug_flag)
+        log_msg ("\t-> Ignored (forbidden by securenets)");
+      return FALSE;
     default:
       *result = TRUE;
       break;
@@ -141,7 +152,7 @@ ypproc_match_2_svc (ypreq_key *argp, ypresp_val *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
       log_msg ("ypproc_match(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
@@ -162,7 +173,7 @@ ypproc_match_2_svc (ypreq_key *argp, ypresp_val *result,
 	case -1:
 	  if (debug_flag)
 	    log_msg ("\t-> Ignored (not a valid source host)");
-	  result->stat = YP_YPERR;
+	  result->stat = YP_NOMAP;
 	  break;
 	case -2:
 	  if (debug_flag)
@@ -174,6 +185,11 @@ ypproc_match_2_svc (ypreq_key *argp, ypresp_val *result,
 	    log_msg ("\t-> Ignored (not a valid domain)");
 	  result->stat = YP_NODOM;
 	  break;
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->stat = YP_NOMAP;
+	  break;
 	}
       return TRUE;
     }
@@ -229,7 +245,7 @@ ypproc_first_2_svc (ypreq_nokey *argp, ypresp_key_val *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_first(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
 
@@ -247,7 +263,8 @@ ypproc_first_2_svc (ypreq_nokey *argp, ypresp_key_val *result,
 	case -1:
 	  if (debug_flag)
 	    log_msg ("\t-> Ignored (not a valid source host)");
-	  result->stat = YP_YPERR;
+	  result->stat = YP_NOMAP;
+	  break;
 	case -2:
           if (debug_flag)
             log_msg ("\t-> Ignored (not a valid map name)");
@@ -258,6 +275,11 @@ ypproc_first_2_svc (ypreq_nokey *argp, ypresp_key_val *result,
             log_msg ("\t-> Ignored (not a valid domain)");
           result->stat = YP_NODOM;
 	  break;
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->stat = YP_NOMAP;
+	  break;
         }
       return TRUE;
     }
@@ -324,7 +346,7 @@ ypproc_next_2_svc (ypreq_key *argp, ypresp_key_val *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
       log_msg ("ypproc_next(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
@@ -346,7 +368,7 @@ ypproc_next_2_svc (ypreq_key *argp, ypresp_key_val *result,
 	case -1:
           if (debug_flag)
             log_msg ("\t-> Ignored (not a valid source host)");
-          result->stat = YP_YPERR;
+          result->stat = YP_NOMAP;
 	  break;
 	case -2:
           if (debug_flag)
@@ -358,6 +380,11 @@ ypproc_next_2_svc (ypreq_key *argp, ypresp_key_val *result,
             log_msg ("\t-> Ignored (not a valid domain)");
           result->stat = YP_NODOM;
 	  break;
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->stat = YP_NOMAP;
+	  break;
         }
       return TRUE;
     }
@@ -421,7 +448,7 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
 		  struct svc_req *rqstp)
 {
   DB_FILE dbp;
-  struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+  const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
   int valid;
 
   if (debug_flag)
@@ -446,6 +473,14 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
     {
       switch (valid)
 	{
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  else
+	    log_msg ("refuse to transfer map from %s",
+		     inet_ntoa (rqhost->sin_addr));
+	  result->xfrstat = YPXFR_REFUSED;
+	  break;
 	case -1:
 	  if (debug_flag)
 	    log_msg ("\t-> Ignored (not a valid source host)");
@@ -506,7 +541,7 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
           if ((size_t)val.dsize != strlen (argp->map_parms.peer) ||
               strncmp (val.dptr, argp->map_parms.peer, val.dsize) != 0)
             {
-	      char buf[val.dsize + 1];
+	      char *buf = alloca (val.dsize + 1);
 
 	      strncpy (buf, val.dptr, val.dsize);
 	      buf[val.dsize] = '\0';
@@ -586,9 +621,9 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
         dup (i);
 
         sprintf (ypxfr_command, "%s/ypxfr", YPBINDIR);
-        sprintf (t, "%u", argp->transid);
-        sprintf (g, "%u", argp->prog);
-        sprintf (p, "%u", argp->port);
+        snprintf (t, sizeof (t), "%u", argp->transid);
+        snprintf (g, sizeof (g), "%u", argp->prog);
+        snprintf (p, sizeof (p), "%u", argp->port);
         if (debug_flag)
           execl (ypxfr_command, "ypxfr", "--debug", "-d",
                  argp->map_parms.domain, "-h", argp->map_parms.peer,
@@ -614,13 +649,12 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
   return TRUE;
 }
 
-bool_t ypproc_clear_2_svc (void *argp __attribute__ ((unused)),
-			   void *result __attribute__ ((unused)),
+bool_t ypproc_clear_2_svc (void *argp UNUSED, void *result UNUSED,
 			   struct svc_req *rqstp)
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_clear_2_svc() [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
     }
@@ -722,7 +756,7 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost;
+      const struct sockaddr_in *rqhost;
 
       rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_all_2_svc(): [From: %s:%d]",
@@ -737,15 +771,24 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
   xdr_ypall_cb.u.close = NULL;
   xdr_ypall_cb.data = NULL;
 
+  /* Set this to TRUE so that the client will be forced to read
+     at least one record from us. This could be the error code.  */
+  result->more = TRUE;
+
   valid = is_valid (rqstp, argp->map, argp->domain);
   if (valid < 1)
     {
       switch (valid)
 	{
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->ypresp_all_u.val.stat = YP_NOMAP;
+	  break;
 	case -1:
 	  if (debug_flag)
 	    log_msg ("\t-> Ignored (not a valid source host)");
-	  result->ypresp_all_u.val.stat = YP_YPERR;
+	  result->ypresp_all_u.val.stat = YP_NOMAP;
 	  break;
 	case -2:
 	  if (debug_flag)
@@ -760,65 +803,33 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
       return TRUE;
     }
 
-  if (children >= MAX_CHILDREN)
+  switch (fork ())
     {
-      int wait = 0;
-
-      while (wait < 3)
-        {
-          sleep (1);
-
-          if (children < MAX_CHILDREN)
-            break;
-          ++wait;
-        }
-    }
-
-  if (children < MAX_CHILDREN)
-    {
-      ++children;
-      switch (fork ())
-        {
-        case 0:
-          ++forked;
+    case 0: /* child */
 #ifdef DEBUG
-          log_msg ("ypserv has forked for ypproc_all(): pid=%i", getpid ());
-          if (!forked)
-            abort ();
+      log_msg ("ypserv has forked for ypproc_all(): pid=%i", getpid ());
 #endif
-          /* Close all databases ! */
-          ypdb_close_all ();
-          break;
-        case -1:
-          --children;
-          log_msg ("WARNING(ypproc_all_2_svc): cannot fork: %s",
-		   strerror (errno));
-          result->ypresp_all_u.val.stat = YP_YPERR;
-          return TRUE;
-        default:
-          return FALSE;
-          break;
-        }
-    }
-  else
-    {
-      log_msg ("WARNING(ypproc_all_2_svc): too many running children!");
+      break;
+    case -1:  /* parent, error */
+      log_msg ("WARNING(ypproc_all_2_svc): cannot fork: %s",
+	       strerror (errno));
       result->ypresp_all_u.val.stat = YP_YPERR;
       return TRUE;
+    default: /* parent, default */
+      return FALSE;
+      break;
     }
 
   /* We are now in the child part. Don't let the child ypserv share
      DB handles with the parent process.  */
   ypdb_close_all();
 
-  result->more = TRUE;
-
   if ((data = calloc (1, sizeof (struct ypall_data))) == NULL)
     {
       log_msg ("ERROR: could not allocate enough memory! [%s|%d]",
 	       __FILE__, __LINE__);
       result->ypresp_all_u.val.stat = YP_YPERR;
-      return TRUE;
+      goto out;
     }
 
   data->dbm = ypdb_open (argp->domain, argp->map);
@@ -857,9 +868,9 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
 	    log_msg ("\t -> First value returned.");
 
 	  if (result->ypresp_all_u.val.stat == YP_TRUE)
-	    return TRUE; /* We return to commit the data.
-			    This also means, we don't give
-			    data free here */
+	    goto out; /* We return to commit the data.
+			 This also means, we don't give
+			 data free here */
 	}
       else
 	result->ypresp_all_u.val.stat = YP_NOMORE;
@@ -872,7 +883,12 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
   if (debug_flag)
     log_msg ("\t -> Exit from ypproc_all without sending data.");
 
-  return TRUE;
+ out:
+  if (!svc_sendreply (rqstp->rq_xprt, (xdrproc_t) xdr_ypresp_all,
+		      (caddr_t) result))
+    svcerr_systemerr (rqstp->rq_xprt);
+  /* Note: no need to free args; we're exiting.  */
+  _exit(0);
 }
 
 bool_t
@@ -884,7 +900,7 @@ ypproc_master_2_svc (ypreq_nokey *argp, ypresp_master *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_master_2_svc(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
 
@@ -902,7 +918,7 @@ ypproc_master_2_svc (ypreq_nokey *argp, ypresp_master *result,
 	case -1:
           if (debug_flag)
             log_msg ("\t-> Ignored (not a valid source host)");
-	  result->stat = YP_YPERR;
+	  result->stat = YP_NOMAP;
 	  break;
 	case -2:
           if (debug_flag)
@@ -913,6 +929,12 @@ ypproc_master_2_svc (ypreq_nokey *argp, ypresp_master *result,
           if (debug_flag)
             log_msg ("\t-> Ignored (not a domain)");
           result->stat = YP_NODOM;
+	  break;
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->stat = YP_NOMAP;
+	  break;
         }
       result->peer = strdup ("");
       return TRUE;
@@ -991,7 +1013,7 @@ ypproc_order_2_svc (ypreq_nokey *argp, ypresp_order *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost;
+      const struct sockaddr_in *rqhost;
 
       rqhost = svc_getcaller (rqstp->rq_xprt);
 
@@ -1012,7 +1034,7 @@ ypproc_order_2_svc (ypreq_nokey *argp, ypresp_order *result,
 	case -1:
           if (debug_flag)
             log_msg ("\t-> Ignored (not a valid source host)");
-          result->stat = YP_YPERR;
+          result->stat = YP_NOMAP;
 	  break;
 	case -2:
           if (debug_flag)
@@ -1024,6 +1046,11 @@ ypproc_order_2_svc (ypreq_nokey *argp, ypresp_order *result,
             log_msg ("\t-> Ignored (not a valid domain)");
           result->stat = YP_NODOM;
 	  break;
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->stat = YP_NOMAP;
+	  break;
         }
       return TRUE;
     }
@@ -1115,7 +1142,7 @@ ypproc_maplist_2_svc (domainname *argp, ypresp_maplist *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
       log_msg ("ypproc_maplist_2_svc(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
@@ -1130,10 +1157,15 @@ ypproc_maplist_2_svc (domainname *argp, ypresp_maplist *result,
     {
       switch (valid)
 	{
+	case 0:
+	  if (debug_flag)
+	    log_msg ("\t-> Ignored (forbidden by securenets)");
+	  result->stat = YP_NOMAP;
+	  break;
 	case -1:
           if (debug_flag)
             log_msg ("\t-> Ignored (not a valid source host)");
-          result->stat = YP_YPERR;
+          result->stat = YP_NOMAP;
 	  break;
 	case -2: /* should never happen */
 	case -3:
@@ -1198,7 +1230,7 @@ ypproc_maplist_2_svc (domainname *argp, ypresp_maplist *result,
 }
 
 int
-ypprog_2_freeresult (SVCXPRT *transp __attribute__ ((unused)),
+ypprog_2_freeresult (SVCXPRT *transp UNUSED,
 		     xdrproc_t xdr_result, caddr_t result)
 {
   xdr_free (xdr_result, result);
