@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2001, 2002, 2003  Thorsten Kukuk
+/* Copyright (c) 2000, 2001, 2002, 2003, 2004  Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -46,6 +46,9 @@
 #include "ypxfr.h"
 #include "ypxfrd.h"
 #include <rpcsvc/ypclnt.h>
+#if defined(HAVE_RPC_CLNT_SOC_H)
+#include <rpc/clnt_soc.h> /* for clntudp_create() */
+#endif /* HAVE_RPC_CLNT_SOC_H */
 #include "compat.h"
 
 #if defined(HAVE_LIBGDBM)
@@ -422,10 +425,12 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
   if (ypproc_master_2 (&req_nokey, &resp_master, clnt_udp) != RPC_SUCCESS)
     {
       log_msg (clnt_sperror (clnt_udp, "ypproc_master_2"));
+      clnt_destroy (clnt_udp);
       return YPXFR_YPERR;
     }
   else if (resp_master.stat != YP_TRUE)
     {
+      clnt_destroy (clnt_udp);
       switch (resp_master.stat)
 	{
 	case YP_NOMAP:
@@ -463,6 +468,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
     }
   else if (resp_order.stat != YP_TRUE)
     {
+      clnt_destroy (clnt_udp);
       switch (resp_order.stat)
 	{
 	case YP_NOMAP:
@@ -547,7 +553,8 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 
   /* Try to use ypxfrd for getting the new map. If it fails, use the old
      method. */
-  if ((result = ypxfrd_transfer (master_host, map, target_domain, dbName_temp)) != 0)
+  if ((result = ypxfrd_transfer (master_host, map,
+				 target_domain, dbName_temp)) != 0)
     {
       /* No success with ypxfrd, get the map entry by entry */
       char orderNum[255];
@@ -573,6 +580,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
         {
           ypdb_close (dbm);
           unlink (dbName_temp);
+	  clnt_destroy (clnt_udp);
           return YPXFR_DBM;
         }
       snprintf (orderNum, sizeof (orderNum), "%ld", (long)masterOrderNum);
@@ -584,6 +592,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
         {
           ypdb_close (dbm);
           unlink (dbName_temp);
+	  clnt_destroy (clnt_udp);
           return YPXFR_DBM;
         }
 
@@ -617,8 +626,9 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
                   unlink (dbName_temp);
                   return YPXFR_DBM;
                 }
+	      xdr_free ((xdrproc_t) xdr_ypresp_val,
+			(char *) &resp_val);
             }
-          xdr_free ((xdrproc_t) xdr_ypresp_val, (char *) &resp_val);
         }
 
       /* Get the YP_SECURE field. */
@@ -649,8 +659,9 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
                   unlink (dbName_temp);
                   return YPXFR_DBM;
                 }
+	      xdr_free ((xdrproc_t) xdr_ypresp_val,
+			(char *) &resp_val);
             }
-          xdr_free ((xdrproc_t) xdr_ypresp_val, (char *) &resp_val);
         }
 
       /* We don't need clnt_udp any longer, give it free */
@@ -739,6 +750,8 @@ main (int argc, char **argv)
   else
     debug_flag = 1;
 
+  memset (&remote_addr, 0, sizeof (remote_addr));
+
   while (1)
     {
       int c;
@@ -817,10 +830,18 @@ main (int argc, char **argv)
   argv += optind;
 
   if (target_domain == NULL)
-    yp_get_default_domain (&target_domain);
+    {
+      yp_get_default_domain (&target_domain);
+
+      if (target_domain == NULL)
+	{
+	  log_msg ("Please specify domainname");
+	  exit (1);
+	}
+    }
 
   if (source_domain == NULL)
-    source_domain = target_domain;
+      source_domain = target_domain;
 
   for (; *argv; argv++)
     {
